/**
 * Known potential responses from the remote API.
 *
 * https://haveibeenpwned.com/api/v3#PwnedPasswords
 *
 */
/** @internal */
export const BAD_REQUEST = {
    status: 400,
    body: 'The hash prefix was not in a valid format',
};
//# sourceMappingURL=responses.js.map