import type { Context, Recogniser } from '.';
import { type EncodingName, type Match } from '../match';
export default class Utf8 implements Recogniser {
    name(): EncodingName;
    match(det: Context): Match | null;
}
