declare const getCodePointsLength: (input: string) => number;
declare const isFullWidth: (x: number) => boolean;
declare const isWideNotCJKTNotEmoji: (x: number) => boolean;
export { getCodePointsLength, isFullWidth, isWideNotCJKTNotEmoji };
