declare const Handlers: {
    isChangeErrorOk: (error: unknown) => boolean;
    isNodeError: (error: unknown) => error is NodeJS.ErrnoException;
    isRetriableError: (error: unknown) => boolean;
    onChangeError: (error: unknown) => undefined;
};
export default Handlers;
