import Conf from 'conf';
import { pkg } from './utils/pkg.js';
export const config = new Conf({
    projectName: pkg.name,
    schema: {
        apiKey: { type: 'string' },
    },
});
//# sourceMappingURL=config.js.map