type TruncationOptions = {
    limit?: number;
    ellipsis?: string;
    ellipsisWidth?: number;
};
type WidthOptions = {
    controlWidth?: number;
    tabWidth?: number;
    emojiWidth?: number;
    regularWidth?: number;
    wideWidth?: number;
};
type Result = {
    width: number;
    index: number;
    truncated: boolean;
    ellipsed: boolean;
};
export type { TruncationOptions, WidthOptions, Result };
