import type { Callable, CallableWithOverloadsWithAlternativeReturn, AttemptifySyncOptions } from './types.js';
declare const attemptifySync: <T extends Callable, U>(fn: T, options: AttemptifySyncOptions<U>) => CallableWithOverloadsWithAlternativeReturn<T, U>;
export default attemptifySync;
