export declare function baseFetch({ baseUrl, endpoint, headers, timeoutMs, signal, userAgent, queryParams, }: {
    baseUrl: string;
    endpoint: string;
    headers?: Record<string, string>;
    timeoutMs?: number;
    signal?: AbortSignal;
    userAgent?: string;
    queryParams?: Record<string, string>;
}): Promise<Response>;
export declare function buildUrl(baseUrl: string, endpoint: string, queryParams?: Record<string, string>): string;
export declare function buildHeaders(userAgent?: string, extra?: Record<string, string>): Record<string, string>;
