export class CreateFileError extends Error {
    name = 'CreateFileError';
    originalError;
    constructor(originalError) {
        super(`Failed to create temporary file.${originalError instanceof Error ? ` ${originalError.message}` : ''}`, { cause: originalError });
        this.originalError = originalError;
    }
}
export class LaunchEditorError extends Error {
    name = 'LaunchEditorError';
    originalError;
    constructor(originalError) {
        super(`Failed to launch editor.${originalError instanceof Error ? ` ${originalError.message}` : ''}`, { cause: originalError });
        this.originalError = originalError;
    }
}
export class ReadFileError extends Error {
    name = 'ReadFileError';
    originalError;
    constructor(originalError) {
        super(`Failed to read temporary file.${originalError instanceof Error ? ` ${originalError.message}` : ''}`, { cause: originalError });
        this.originalError = originalError;
    }
}
export class RemoveFileError extends Error {
    name = 'RemoveFileError';
    originalError;
    constructor(originalError) {
        super(`Failed to remove temporary file.${originalError instanceof Error ? ` ${originalError.message}` : ''}`, { cause: originalError });
        this.originalError = originalError;
    }
}
