import type { Callable, CallableWithOverloadsWithAlternativeReturn, AttemptifyAsyncOptions } from './types.js';
declare const attemptifyAsync: <T extends Callable, U>(fn: T, options: AttemptifyAsyncOptions<U>) => CallableWithOverloadsWithAlternativeReturn<T, U>;
export default attemptifyAsync;
