import type { Callable, RetryifyAsyncOptions, RetryifyAsyncCallOptions } from './types.js';
declare const retryifyAsync: <T extends Callable>(fn: T, options: RetryifyAsyncOptions) => ((options: RetryifyAsyncCallOptions) => T);
export default retryifyAsync;
