import type { WidthOptions as Options } from 'fast-string-truncated-width';
declare const fastStringWidth: (input: string, options?: Options) => number;
export default fastStringWidth;
export type { Options };
