import type { Callable, RetryifySyncOptions, RetryifySyncCallOptions } from './types.js';
declare const retryifySync: <T extends Callable>(fn: T, options: RetryifySyncOptions) => ((options: RetryifySyncCallOptions) => T);
export default retryifySync;
