import type { TruncationOptions, WidthOptions, Result } from './types.js';
declare const getStringTruncatedWidth: (input: string, truncationOptions?: TruncationOptions, widthOptions?: WidthOptions) => Result;
export default getStringTruncatedWidth;
export type { TruncationOptions, WidthOptions, Result };
