import type { Exception } from '../types.js';
declare const isException: (value: unknown) => value is Exception;
declare const isFunction: (value: unknown) => value is Function;
declare const isString: (value: unknown) => value is string;
declare const isUndefined: (value: unknown) => value is undefined;
export { isException, isFunction, isString, isUndefined };
