/**
 * Known potential responses from the remote API.
 *
 * Unfortunately, the API does not send a decent human-readable message back with each response, but
 * they are documented on the website: https://haveibeenpwned.com/api/v3#ResponseCodes
 *
 * These objects simply provide a mapping between the HTTP response status code and the
 * corresponding human-readable message so we can throw a more descriptive error for the consumer.
 * (They are also leveraged in our tests.)
 */
export {};
