declare const ATTEMPTIFY_CHANGE_ERROR_OPTIONS: {
    onError: (error: unknown) => undefined;
};
declare const ATTEMPTIFY_NOOP_OPTIONS: {
    onError: () => undefined;
};
declare const IS_USER_ROOT: boolean;
declare const RETRYIFY_OPTIONS: {
    isRetriable: (error: unknown) => boolean;
};
export { ATTEMPTIFY_CHANGE_ERROR_OPTIONS, ATTEMPTIFY_NOOP_OPTIONS, IS_USER_ROOT, RETRYIFY_OPTIONS };
