export declare class CreateFileError extends Error {
    name: string;
    originalError: unknown;
    constructor(originalError: unknown);
}
export declare class LaunchEditorError extends Error {
    name: string;
    originalError: unknown;
    constructor(originalError: unknown);
}
export declare class ReadFileError extends Error {
    name: string;
    originalError: unknown;
    constructor(originalError: unknown);
}
export declare class RemoveFileError extends Error {
    name: string;
    originalError: unknown;
    constructor(originalError: unknown);
}
